package be.iminds.ilabt.jfed.git;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.io.File;
import java.nio.file.Files;

public class GitFetcherTest {

    private static final Logger LOG = LoggerFactory.getLogger(GitFetcherTest.class);
    private static final String PRIVATE_REPO_SSH_URL = "git@github.com:twalcari/Hello-World.git";
    private static final String PRIVATE_REPO_HTTPS_URL = "https://github.com/twalcari/Hello-World.git";

    private File checkoutDirSimple;
    private File checkoutDirPrivateSsh;
    private File checkoutDirPrivateHttps;
    @BeforeMethod
    public void setUp() throws Exception {
        checkoutDirSimple = Files.createTempDirectory("gitfetcherSimpleTest").toFile();
        checkoutDirSimple.deleteOnExit();

        checkoutDirPrivateSsh = Files.createTempDirectory("gitfetcherPrivateSshTest").toFile();
        checkoutDirPrivateSsh.deleteOnExit();

        checkoutDirPrivateHttps = Files.createTempDirectory("gitfetcherPrivateHttpsTest").toFile();
        checkoutDirPrivateHttps.deleteOnExit();
    }


    @Test
    public void testFetchSimple() throws Exception {

        GitFetcherCommandBuilder commandBuilder = new GitFetcherCommandBuilder();
        LOG.debug("Cloning to {}", checkoutDirSimple);

        commandBuilder.setTargetDir(checkoutDirSimple);
        commandBuilder.setGitUrl("https://github.com/octocat/Hello-World.git");

        GitFetcher gitFetcher = new GitFetcher(null);
        gitFetcher.fetch(commandBuilder.build());

        Assert.assertTrue(new File(checkoutDirSimple, "README").exists(), "Could not find expected file");
    }

    @Test(enabled = false) //disabled because I think this one needs real credentials too: Exception fetching git repo:git@github.com:twalcari/Hello-World.git: remote hung up unexpectedly
    public void testFetchPrivateSsh() throws Exception {

        GitFetcherCommandBuilder commandBuilder = new GitFetcherCommandBuilder();
        LOG.debug("Cloning to {}", checkoutDirPrivateSsh);

        commandBuilder.setTargetDir(checkoutDirPrivateSsh);
        commandBuilder.setGitUrl(PRIVATE_REPO_SSH_URL);
        commandBuilder.setPrivateKeyPemFile(getClass().getResource("gitfetcher-test.pem").getFile());

        GitFetcher gitFetcher = new GitFetcher(null);
        gitFetcher.fetch(commandBuilder.build());

        Assert.assertTrue(new File(checkoutDirPrivateSsh, "README").exists(), "Could not find expected file");
    }

    /**
     * This test is disabled because it needs real credentials. It was tested successfully on https://github.com/twalcari/Hello-World.git
     * @throws Exception
     */
    @Test(enabled = false)
    public void testFetchPrivateHttps() throws Exception {

        GitFetcherCommandBuilder commandBuilder = new GitFetcherCommandBuilder();
        LOG.debug("Cloning to {}", checkoutDirPrivateHttps);

        commandBuilder.setTargetDir(checkoutDirPrivateHttps);
        commandBuilder.setGitUrl(PRIVATE_REPO_HTTPS_URL);
        commandBuilder.setUsername("twalcari");
        commandBuilder.setPassword("XXXXXX");

        GitFetcher gitFetcher = new GitFetcher(null);
        gitFetcher.fetch(commandBuilder.build());

        Assert.assertTrue(new File(checkoutDirPrivateHttps, "README").exists(), "Could not find expected file");
    }


    @AfterMethod
    public void tearDown() throws Exception {
        checkoutDirSimple.delete();
        checkoutDirPrivateSsh.delete();
        checkoutDirPrivateHttps.delete();
    }

}